@@ -1,7 +1,7 @@
 /* Trampoline construction */
 
 /*
- * Copyright 1995-1999, 2001-2002 Bruno Haible, <haible@clisp.cons.org>
+ * Copyright 1995-1999, 2001-2005 Bruno Haible, <bruno@clisp.org>
  *
  * This is free software distributed under the GNU General Public Licence
  * described in the file COPYING. Contact the author if you don't have this
@@ -22,7 +22,7 @@
 #endif
 #if defined(__rs6000__)
 #if defined(__NetBSD__)
-#define __powerpcnetbsd__
+#define __rs6000netbsd__
 #else
 #if !defined(_AIX)
 #define __rs6000sysv4__  /* SysV.4 ABI, real machine code. */
@@ -95,50 +95,17 @@ extern void (*tramp_r) (); /* trampoline prototype */
 #include <stdio.h> /* declares fprintf() */
 
 #include <sys/types.h>
-#ifdef HAVE_STDLIB_H
-#include <stdlib.h>
-#endif
+#include <stdlib.h> /* declares abort(), malloc(), free() */
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
-/* Declare malloc(), free(). */
-#ifndef malloc
-#ifdef __cplusplus
-extern "C" RETMALLOCTYPE malloc (MALLOC_SIZE_T size);
-#elif defined(__STDC__)
-extern RETMALLOCTYPE malloc (MALLOC_SIZE_T size);
-#else
-extern RETMALLOCTYPE malloc();
-#endif
-#endif
-#ifndef free
-#ifdef __cplusplus
-extern "C" RETFREETYPE free (RETMALLOCTYPE ptr);
-#elif defined(__STDC__)
-extern RETFREETYPE free (RETMALLOCTYPE ptr);
-#else
-extern RETFREETYPE free();
-#endif
-#endif
-
-/* Declare abort(). */
-#ifdef __cplusplus
-extern "C" ABORT_VOLATILE RETABORTTYPE abort (void);
-#elif defined(__STDC__)
-extern ABORT_VOLATILE RETABORTTYPE abort (void);
-#else
-extern ABORT_VOLATILE RETABORTTYPE abort();
-#endif
-
 /* Declare getpagesize(). */
 #ifdef HAVE_GETPAGESIZE
 #ifdef __cplusplus
 extern "C" RETGETPAGESIZETYPE getpagesize (void);
-#elif defined(__STDC__)
-extern RETGETPAGESIZETYPE getpagesize (void);
 #else
-extern RETGETPAGESIZETYPE getpagesize ();
+extern RETGETPAGESIZETYPE getpagesize (void);
 #endif
 #else
 #ifdef HAVE_SYS_PARAM_H
@@ -169,15 +136,6 @@ extern RETGETPAGESIZETYPE getpagesize ();
 #else
 #include <sys/types.h>
 #include <sys/mman.h>
-#if !defined(__convex__)
-#ifdef __cplusplus
-extern "C" int mprotect (MPROTECT_CONST MMAP_ADDR_T addr, MMAP_SIZE_T len, int prot);
-#elif defined(__STDC__)
-extern int mprotect (MPROTECT_CONST MMAP_ADDR_T addr, MMAP_SIZE_T len, int prot);
-#else
-extern int mprotect ();
-#endif
-#endif
 #endif
 #endif
 #endif
@@ -189,13 +147,6 @@ extern int mprotect ();
 #if !defined(PROT_EXEC) && defined(PROT_EXECUTE) /* Irix 4.0.5 needs this */
 #define PROT_EXEC PROT_EXECUTE
 #endif
-#ifdef __cplusplus
-extern "C" RETMMAPTYPE mmap (MMAP_ADDR_T addr, MMAP_SIZE_T len, int prot, int flags, int fd, off_t off);
-#elif defined(__STDC__)
-extern RETMMAPTYPE mmap (MMAP_ADDR_T addr, MMAP_SIZE_T len, int prot, int flags, int fd, off_t off);
-#else
-extern RETMMAPTYPE mmap ();
-#endif
 #endif
 
 /* Declare open(). */
@@ -206,21 +157,6 @@ extern RETMMAPTYPE mmap ();
 #ifdef OPEN_NEEDS_SYS_FILE_H
 #include <sys/file.h>
 #endif
-#ifdef __cplusplus
-#ifdef OPEN_DOTS
-extern "C" int open (OPEN_CONST char* path, int flags, ...);
-#else
-extern "C" int open (OPEN_CONST char* path, int flags, mode_t mode);
-#endif
-#elif defined(__STDC__)
-#ifdef OPEN_DOTS
-extern int open (OPEN_CONST char* path, int flags, ...);
-#else
-extern int open (OPEN_CONST char* path, int flags, mode_t mode);
-#endif
-#else
-extern int open ();
-#endif
 #endif
 
 /* Declare shmget(), shmat(), shmctl(). */
@@ -231,27 +167,6 @@ extern int open ();
 #ifdef HAVE_SYS_SYSMACROS_H
 #include <sys/sysmacros.h>
 #endif
-#ifdef __cplusplus
-extern "C" int shmget (key_t key, SHMGET_SIZE_T size, int shmflg);
-extern "C" RETSHMATTYPE shmat (int shmid, SHMAT_CONST RETSHMATTYPE shmaddr, int shmflg);
-#ifdef SHMCTL_DOTS
-extern "C" int shmctl (int shmid, int cmd, ...);
-#else
-extern "C" int shmctl (int shmid, int cmd, struct shmid_ds * buf);
-#endif
-#elif defined(__STDC__)
-extern int shmget (key_t key, SHMGET_SIZE_T size, int shmflg);
-extern RETSHMATTYPE shmat (int shmid, SHMAT_CONST RETSHMATTYPE shmaddr, int shmflg);
-#ifdef SHMCTL_DOTS
-extern int shmctl (int shmid, int cmd, ...);
-#else
-extern int shmctl (int shmid, int cmd, struct shmid_ds * buf);
-#endif
-#else
-extern int shmget ();
-extern RETSHMATTYPE shmat ();
-extern int shmctl ();
-#endif
 #endif
 
 /* Support for instruction cache flush. */
@@ -289,7 +204,7 @@ extern int shmctl ();
 #include <sys/syslocal.h>
 #endif
 /* Inline assembly function for instruction cache flush. */
-#if defined(__sparc__) || defined(__sparc64__) || defined(__alpha__) || defined(__hppaold__) || defined(__rs6000sysv4__) || defined(__convex__) || defined(__powerpcnetbsd__)
+#if defined(__sparc__) || defined(__sparc64__) || defined(__alpha__) || defined(__hppaold__) || defined(__rs6000sysv4__) || defined(__rs6000netbsd__) || defined(__convex__)
 #ifdef __GNUC__
 extern inline
 #if defined(__sparc__) || defined(__sparc64__)
@@ -366,7 +281,11 @@ extern void __TR_clear_cache();
 #define TRAMP_LENGTH 32
 #define TRAMP_ALIGN 4
 #endif
-#if defined(__rs6000sysv4__) || defined(__powerpcnetbsd__)
+#ifdef __rs6000sysv4__
+#define TRAMP_LENGTH 24
+#define TRAMP_ALIGN 4
+#endif
+#ifdef __rs6000netbsd__
 #define TRAMP_LENGTH 24
 #define TRAMP_ALIGN 4
 #endif
@@ -386,6 +305,10 @@ extern void __TR_clear_cache();
 #define TRAMP_LENGTH 32
 #define TRAMP_ALIGN 16
 #endif
+#ifdef __x86_64__
+#define TRAMP_LENGTH 22
+#define TRAMP_ALIGN 16
+#endif
 #ifdef __s390__
 #define TRAMP_LENGTH 22
 #define TRAMP_ALIGN 2
@@ -405,14 +328,7 @@ extern void __TR_clear_cache();
 static char* freelist = NULL;
 #endif
 
-#if defined(__STDC__) || defined(__GNUC__) || defined(__cplusplus)
 __TR_function alloc_trampoline_r (__TR_function address, void* data0, void* data1)
-#else
-__TR_function alloc_trampoline_r (address, data0, data1)
-  __TR_function address;
-  void* data0;
-  void* data1;
-#endif
 {
   char* function;
   char* data;
@@ -594,37 +510,6 @@ __TR_function alloc_trampoline_r (address, data0, data1)
 #define tramp_data(function)  \
   *(unsigned int *) (function +16)
 #endif
-#ifdef __s390__
-  /* function:
-
-        bras    %r1,.LTN0_0
-.LT0_0:
-.LC0:
-        .long   0x73554711
-.LC1:
-        .long   0xbabebec0
-.LTN0_0:
-        l       %r0,.LC0-.LT0_0(%r1)
-        l       %r1,.LC1-.LT1_0(%r1)
-        br      %r1
-  */
-  /* What about big endian / little endian ?? */
-  *(unsigned int *) (function + 0) = 0xA7150006;
-  *(unsigned int *) (function + 4) = (unsigned int) data;
-  *(unsigned int *) (function + 8) = (unsigned int) address;
-  *(unsigned int *) (function +12) = 0x58001000;
-  *(unsigned int *) (function +16) = 0x58101004;
-  *(unsigned short *) (function +20) = 0x07f1;
-#define is_tramp(function)  \
-  *(unsigned int *)    (function + 0) == 0xA7150006 && \
-  *(unsigned int *)    (function +12) == 0x58001000 && \
-  *(unsigned int *)    (function +16) == 0x58101004 && \
-  *(unsigned short *)  (function +20) == 0x07f1
-#define tramp_address(function)  \
-  *(unsigned int *) (function +8)
-#define tramp_data(function)  \
-  *(unsigned int *) (function +4)
-#endif
 #ifdef __mips64old__
   /* function:
    *    dli $2,<data>			3C 02 hi16(hi32(<data>))
@@ -871,14 +756,14 @@ __TR_function alloc_trampoline_r (address, data0, data1)
    *    .long   <address>
    */
   { /* work around a bug in gcc 3.* */
-    void *foo=&tramp_r;
-    *(long *) (function + 0) = ((long *) ((char*)foo-2))[0];
+    void* tramp_r_address = &tramp_r;
+    *(long *) (function + 0) = ((long *) ((char*)tramp_r_address-2))[0];
     *(long *) (function + 4) = (long) (function + 8);
     *(long *) (function + 8) = (long) data;
     *(long *) (function +12) = (long) address;
   }
 #define is_tramp(function)  \
-  { void *foo=&tramp_r; *((long *) function) == *((long *) ((char*)foo-2)); }
+  ((long *) function)[0] == ((long *) ((char*)tramp_r_address-2))[0]
 #define tramp_address(function)  \
   ((long *) function)[3]
 #define tramp_data(function)  \
@@ -951,7 +836,7 @@ __TR_function alloc_trampoline_r (address, data0, data1)
 #define tramp_data(function)  \
   hilo(*(unsigned short *) (function + 2), *(unsigned short *) (function + 6))
 #endif
-#ifdef __powerpcnetbsd__
+#ifdef __rs6000netbsd__
   /* function:
    *    {liu|lis} 13,hi16(<data>)		3D A0 hi16(<data>)
    *    {oril|ori} 13,13,lo16(<data>)		61 AD lo16(<data>)
@@ -1073,6 +958,67 @@ __TR_function alloc_trampoline_r (address, data0, data1)
   ((long *) function)[2]
 #define tramp_data(function)  \
   ((long *) function)[3]
+#endif
+#ifdef __x86_64__
+  /* function:
+   *    movabsq $<data>,%r10		49 BA <data>
+   *    movabsq $<address>,%rax		48 B8 <address>
+   *    jmp *%rax			FF E0
+   */
+  *(short *) (function + 0) = 0xBA49;
+  *(short *) (function + 2) = (unsigned long) data & 0xffff;
+  *(int *)   (function + 4) = ((unsigned long) data >> 16) & 0xffffffff;
+  *(short *) (function + 8) = ((unsigned long) data >> 48) & 0xffff;
+  *(short *) (function +10) = 0xB848;
+  *(int *)   (function +12) = (unsigned long) address & 0xffffffff;
+  *(int *)   (function +16) = ((unsigned long) address >> 32) & 0xffffffff;
+  *(short *) (function +20) = 0xE0FF;
+#define is_tramp(function)  \
+  *(unsigned short *) (function + 0) == 0xBA49 && \
+  *(unsigned short *) (function +10) == 0xB848 && \
+  *(unsigned short *) (function +20) == 0xE0FF
+#define hilo(hiword,loword)  \
+  (((unsigned long) (hiword) << 32) | (unsigned long) (loword))
+#define himidlo(hishort,midword,loshort)  \
+  (((unsigned long) (hishort) << 48) | (unsigned long) (midword) << 16 \
+   | (unsigned long) (loshort))
+#define tramp_address(function)  \
+  hilo(*(unsigned int *) (function +16), *(unsigned int *) (function +12))
+#define tramp_data(function)  \
+  himidlo(*(unsigned short *) (function + 8), \
+          *(unsigned int *)   (function + 4), \
+          *(unsigned short *) (function + 2))
+#endif
+#ifdef __s390__
+  /* function:
+
+        bras    %r1,.LTN0_0
+.LT0_0:
+.LC0:
+        .long   0x73554711
+.LC1:
+        .long   0xbabebec0
+.LTN0_0:
+        l       %r0,.LC0-.LT0_0(%r1)
+        l       %r1,.LC1-.LT1_0(%r1)
+        br      %r1
+  */
+  /* What about big endian / little endian ?? */
+  *(unsigned int *)   (function + 0) = 0xA7150006;
+  *(unsigned int *)   (function + 4) = (unsigned int) data;
+  *(unsigned int *)   (function + 8) = (unsigned int) address;
+  *(unsigned int *)   (function +12) = 0x58001000;
+  *(unsigned int *)   (function +16) = 0x58101004;
+  *(unsigned short *) (function +20) = 0x07f1;
+#define is_tramp(function)  \
+  *(int *)            (function + 0) == 0xA7150006 && \
+  *(int *)            (function +12) == 0x58001000 && \
+  *(int *)            (function +16) == 0x58101004 && \
+  *(short *)          (function +20) == 0x07f1
+#define tramp_address(function)  \
+  *(unsigned int *) (function +8)
+#define tramp_data(function)  \
+  *(unsigned int *) (function +4)
 #endif
   /*
    * data:
@@ -1102,7 +1048,7 @@ __TR_function alloc_trampoline_r (address, data0, data1)
 #if defined(HAVE_SYS_M88KBCS_H)
     if (memctl(start_addr, len, MCT_TEXT) == -1)
 #else
-    if (mprotect((MMAP_ADDR_T)start_addr, len, PROT_READ|PROT_WRITE|PROT_EXEC) < 0)
+    if (mprotect((void*)start_addr, len, PROT_READ|PROT_WRITE|PROT_EXEC) < 0)
 #endif
 #endif
 #endif
@@ -1213,12 +1159,7 @@ __TR_function alloc_trampoline_r (address, data0, data1)
   return (__TR_function) (function + TRAMP_BIAS);
 }
 
-#if defined(__STDC__) || defined(__GNUC__) || defined(__cplusplus)
 void free_trampoline_r (__TR_function function)
-#else
-void free_trampoline_r (function)
-  __TR_function function;
-#endif
 {
 #if TRAMP_BIAS
   function = (__TR_function)((char*)function - TRAMP_BIAS);
@@ -1231,15 +1172,11 @@ void free_trampoline_r (function)
 #endif
 }
 
-#if defined(__STDC__) || defined(__GNUC__) || defined(__cplusplus)
 int is_trampoline_r (void* function)
-#else
-int is_trampoline_r (function)
-  void* function;
-#endif
 {
 #if defined(is_tramp) && defined(tramp_data)
 #ifdef __hppanew__
+  void* tramp_r_address = &tramp_r;
   if (!(((long)function & 3) == (TRAMP_BIAS & 3))) return 0;
 #endif
   return
@@ -1252,12 +1189,7 @@ int is_trampoline_r (function)
 #endif
 }
 
-#if defined(__STDC__) || defined(__GNUC__) || defined(__cplusplus)
 __TR_function trampoline_r_address (void* function)
-#else
-__TR_function trampoline_r_address (function)
-  void* function;
-#endif
 {
 #ifdef tramp_address
   return (__TR_function)(tramp_address(((char*)function - TRAMP_BIAS)));
@@ -1266,12 +1198,7 @@ __TR_function trampoline_r_address (function)
 #endif
 }
 
-#if defined(__STDC__) || defined(__GNUC__) || defined(__cplusplus)
 void* trampoline_r_data0 (void* function)
-#else
-void* trampoline_r_data0 (function)
-  void* function;
-#endif
 {
 #ifdef tramp_data
   return ((void**)((char*)function-TRAMP_BIAS+TRAMP_LENGTH))[0];
@@ -1280,12 +1207,7 @@ void* trampoline_r_data0 (function)
 #endif
 }
 
-#if defined(__STDC__) || defined(__GNUC__) || defined(__cplusplus)
 void* trampoline_r_data1 (void* function)
-#else
-void* trampoline_r_data1 (function)
-  void* function;
-#endif
 {
 #ifdef tramp_data
   return ((void**)((char*)function-TRAMP_BIAS+TRAMP_LENGTH))[1];
