@@ -1,33 +1,24 @@
 ;; Foreign functions provided by the Linux C library version 6,
 ;; i.e. the GNU C library version 2.0.7.
 ;; Bruno Haible 10.4.1998, 19.4.1998
-;; Sam Steingold 2002-2003
+;; Sam Steingold 2002-2005
 
 (defpackage "LINUX"
-  (:case-sensitive t)
+  (:modern t)
   (:nicknames "UNIX" "GLIBC")
-  (:use))
+  (:use "COMMON-LISP" "FFI")
+  (:shadowing-import-from "EXPORTING"
+           #:defconstant #:defun #:defmacro #:define-modify-macro
+           #:define-symbol-macro #:defsetf
+           #:def-c-type #:def-c-enum #:def-c-struct #:def-c-var #:def-call-out)
+  (:shadow read write random abort abs acos asin atan cos sin tan cosh sinh tanh
+           acosh asinh atanh exp log sqrt floor truncate ftruncate open close
+           remove sleep))
 
 ;; This requires linking with NEW_LIBS='linux.o -lm'.
 
 (ffi:default-foreign-language :stdc)
 
-(eval-when (compile eval)
-  ;; so that we don't need to prefix everything with "lisp:" or "ffi:".
-  (load "../../exporting")
-  (make-exporting "LINUX"
-    cl:aref cl:ash cl:coerce cl:compile cl:defconstant cl:dotimes cl:eval
-    cl:fill cl:floor cl:gensym cl:let cl:load cl:load-time-value cl:logand
-    cl:logbitp cl:logior cl:lognot cl:mod cl:multiple-value-bind cl:not
-    cl:or cl:progn cl:setf cl:t cl:zerop cl:+ cl:- cl:* cl:= cl:1-
-    ffi:bitsizeof ffi:boolean ffi:cast ffi:char ffi:character ffi:c-array
-    ffi:c-array-max ffi:c-array-ptr ffi:c-function ffi:c-ptr ffi:c-ptr-null
-    ffi:c-pointer ffi:c-string ffi:c-struct ffi:deref ffi::foreign-value
-    ffi:double-float ffi:element ffi:int ffi:long ffi:nil ffi:short ffi:sint8
-    ffi:sint16 ffi:sint32 ffi:sint64 ffi:single-float ffi:sizeof ffi:slot
-    ffi:uchar ffi:uint ffi:uint8 ffi:uint16 ffi:uint32 ffi:uint64 ffi:ulong
-    ffi:ushort ffi:with-c-var))
-
 (in-package "LINUX")
 
 (def-c-type longlong sint64)
@@ -301,28 +292,27 @@
 
 ;; in GNU, errno is a per-thread variable, so def-c-var is not appropriate
 ;; (def-c-var errno (:type ffi:int))
-;; link error: "undefined reference to `errno'"
+;; Instead, errno is a cpp macro expanding to *__errno_location()
+;; However, on PPC/glibc, no __errno_location() was found, and
+;; some people use this file with *BSD.
 
-(c-lines "#include <bits/errno.h>~%")
+(c-lines "static int *clisp_errno_location(void);~%")
+(c-lines "static int *clisp_errno_location() { return &errno; }~%")
 
-(def-call-out __errno_location (:arguments) (:return-type c-pointer))
-(def-call-out get-errno (:name "__errno_location")
+(ffi:def-call-out errno-location (:name "clisp_errno_location")
+  (:arguments) (:return-type (c-pointer int)))
+
+;; (define-symbol-macro errno (foreign-value (__errno_location))); setf-able
+;; Optimize the most common use through an extra function
+(ffi:def-call-out get-errno (:name "clisp_errno_location")
   (:arguments) (:return-type (c-ptr int)))
 
-(define-symbol-macro errno (get-errno)); not setf-able per se
-(defun set-errno (value)
-  ;; not efficient at all, better redone as LISPFUNN
-  (with-c-var (el 'c-pointer (__errno_location))
-    (setf (cast el '(c-ptr int)) value)))
-(defsetf get-errno set-errno)
+(define-symbol-macro errno (get-errno)); not yet setf-able
+(cl:defun set-errno (value)
+  (setf (foreign-value (errno-location)) value))
+(cl:defsetf get-errno set-errno); errno becomes setf-able
 
-;; in some possible future:
-;; (def-call-out __errno_location (:arguments) (:return-type (c-pointer int)))
-;; (define-symbol-macro errno (foreign-value (__errno_location))); setf-able
-
-; ------------------------------ <errno.h> ------------------------------------
-
-; ============================ <sys/errno.h> ==================================
+; ---------------------------- <sys/errno.h> ----------------------------------
 
 ;;; ============================== <float.h> ==================================
 
@@ -610,10 +600,8 @@
   (:return-type long))
 (def-call-out srand48 (:arguments (seedval long))
   (:return-type nil))
-;(def-call-out seed48 (:arguments (seed16v (c-ptr (c-array ushort 3))))
-;  (:return-type (c-ptr (c-array ushort 3)) :none))
 (def-call-out seed48 (:arguments (seed16v (c-ptr (c-array ushort 3))))
-  (:return-type (c-ptr ushort) :none))
+  (:return-type (c-ptr (c-array ushort 3)) :none))
 (def-call-out lcong48 (:arguments (param (c-ptr (c-array ushort 7))))
   (:return-type nil))
 
@@ -1059,15 +1047,18 @@
 
 (def-call-out close (:arguments (fd int)) (:return-type int))
 
-(def-call-out read-helper
-    (:arguments (fd int) (buf c-pointer) (nbytes size_t) (no-hang-p boolean))
-  (:return-type ssize_t) (:name "read_helper"))
-(defmacro read (fd buf nbytes) `(read-helper ,fd ,buf ,nbytes nil))
-(def-call-out write-helper
+(def-c-enum perseverance
+  persev_full persev_partial persev_immediate persev_bonus)
+(def-call-out fd-read
     (:arguments (fd int) (buf (c-ptr (c-array-max char 4096)) :out :alloca)
-                (nbytes size_t) (no-hang-p boolean))
-  (:return-type ssize_t) (:name "write_helper"))
-(defmacro write (fd buf nbytes) `(write-helper ,fd ,buf ,nbytes nil))
+                (nbytes size_t) (persev perseverance))
+  (:return-type ssize_t) (:name "fd_read"))
+(defmacro read (fd buf nbytes) `(fd-read ,fd ,buf ,nbytes persev_full))
+(def-call-out fd-write
+    (:arguments (fd int) (buf c-pointer)
+                (nbytes size_t) (persev perseverance))
+  (:return-type ssize_t) (:name "fd_write"))
+(defmacro write (fd buf nbytes) `(fd-write ,fd ,buf ,nbytes persev_full))
 
 (def-call-out pipe (:arguments (pipedes (c-ptr (c-array int 2)) :out))
   (:return-type int))
@@ -1118,61 +1109,32 @@
     (:arguments (path c-string) (argv (c-array-ptr c-string))
                 (envp (c-array-ptr c-string)))
   (:return-type int)
-  (:name "execv"))
+  (:name "execve"))
 (def-call-out execvp
     (:arguments (file c-string) (argv (c-array-ptr c-string)))
   (:return-type int)
   (:name "execvp"))
 
-(def-call-out execle0
-    (:arguments (path c-string) (argv0 c-string) (null c-string)
-                (envp c-pointer))
-  (:return-type int) (:name "execle"))
-(def-call-out execle1
-    (:arguments (path c-string) (argv0 c-string) (argv1 c-string)
-                (null c-string) (envp c-pointer))
-  (:return-type int) (:name "execle"))
-(def-call-out execle2
-    (:arguments (path c-string) (argv0 c-string) (argv1 c-string)
-                (argv2 c-string) (null c-string) (envp c-pointer))
-  (:return-type int) (:name "execle"))
-(def-call-out execle3
-    (:arguments (path c-string) (argv0 c-string) (argv1 c-string)
-                (argv2 c-string) (argv3 c-string) (null c-string)
-                (envp c-pointer))
-  (:return-type int) (:name "execle"))
-(def-call-out execl0
-    (:arguments (path c-string) (argv0 c-string) (null c-string))
-  (:return-type int) (:name "execl"))
+; Foreign language interfaces should use the execv* series of functions,
+; not the C varargs convenience functions.
+(defun execl (path &rest args)
+  (execv path (coerce args 'vector)))
+(defun execlp (file &rest args)
+  (execvp file (coerce args 'vector)))
+
+; Provide these stubs so the execl/p/e names become registered
 (def-call-out execl1
     (:arguments (path c-string) (argv0 c-string) (argv1 c-string)
                 (null c-string))
   (:return-type int) (:name "execl"))
-(def-call-out execl2
-    (:arguments (path c-string) (argv0 c-string) (argv1 c-string)
-                (argv2 c-string) (null c-string))
-  (:return-type int) (:name "execl"))
-(def-call-out execl3
-    (:arguments (path c-string) (argv0 c-string) (argv1 c-string)
-                (argv2 c-string) (argv3 c-string) (null c-string))
-  (:return-type int) (:name "execl"))
-;(def-call-out execvp (:arguments (path c-string) (argv c-pointer)) ; ??
-;  (:return-type int))
-(def-call-out execlp0
-    (:arguments (path c-string) (argv0 c-string) (null c-string))
-  (:return-type int) (:name "execlp"))
 (def-call-out execlp1
     (:arguments (path c-string) (argv0 c-string) (argv1 c-string)
                 (null c-string))
   (:return-type int) (:name "execlp"))
-(def-call-out execlp2
+(def-call-out execle1
     (:arguments (path c-string) (argv0 c-string) (argv1 c-string)
-                (argv2 c-string) (null c-string))
-  (:return-type int) (:name "execlp"))
-(def-call-out execlp3
-    (:arguments (path c-string) (argv0 c-string) (argv1 c-string)
-                (argv2 c-string) (argv3 c-string) (null c-string))
-  (:return-type int) (:name "execlp"))
+                (null c-string) (envp c-pointer))
+  (:return-type int) (:name "execle"))
 
 (def-call-out nice (:arguments (increment int)) (:return-type int))
 
@@ -2031,6 +1993,7 @@
 
 (c-lines "#include <sys/utsname.h>~%")
 
+
 ;;; ---------------------------- <utsnamelen.h> ------------------------------
 
 (eval-when (load compile eval)
@@ -2387,6 +2350,7 @@ I don't know how to do these...
 But it's not the end of the world, since its easy to accomplish what SIG_DFL
 and SIG_IGN do anyway, by other means ... They are just sugar, really.
 
+
 |#
 ;;; --------------------------------- <bits/sigset.h> -----------------------
 (eval-when (load compile eval)
@@ -2396,7 +2360,6 @@ and SIG_IGN do anyway, by other means ... They are just sugar, really.
 
 (def-c-struct sigset_t
   (val (c-array-max uint #.SIGSET_NWORDS)))
-
 ;;; --------------------------------- <bits/sigaction.h> ---------------------
 
 (def-c-type sighandler_t (c-function (:arguments (sig int)) ;from signal.h
@@ -2494,6 +2457,5 @@ and SIG_IGN do anyway, by other means ... They are just sugar, really.
   (:return-type int))
 
 ;;; ==========================================================================
-;;; clean up
-(cl:in-package "CL-USER")
+
 (provide "linux")
