@@ -22,7 +22,7 @@
 
 (defun wm-name (window)
   (declare (type window window))
-  (declare (values string))
+  (declare (clx-values string))
   (get-property window :WM_NAME :type :STRING :result-type 'string :transform #'card8->char))
 
 (defsetf wm-name (window) (name)
@@ -37,7 +37,7 @@
 
 (defun wm-icon-name (window)
   (declare (type window window))
-  (declare (values string))
+  (declare (clx-values string))
   (get-property window :WM_ICON_NAME :type :STRING
 		:result-type 'string :transform #'card8->char))
 
@@ -46,7 +46,7 @@
 
 (defun wm-client-machine (window)
   (declare (type window window))
-  (declare (values string))
+  (declare (clx-values string))
   (get-property window :WM_CLIENT_MACHINE :type :STRING
 		:result-type 'string :transform #'card8->char))
 
@@ -55,42 +55,45 @@
 
 (defun get-wm-class (window)
   (declare (type window window))
-  (declare (values (or null name-string) (or null class-string)))
-  (let ((value (get-property window :WM_CLASS :type :STRING
-			     :result-type 'string :transform #'card8->char)))
-    (declare (type (or null string) value))
+  (declare (clx-values (or null name-string) (or null class-string)))
+  (let ((value (get-property window :WM_CLASS :type :STRING :result-type '(vector card8))))
+    (declare (type (or null (vector card8)) value))
     (when value
-      (let* ((name-len (position #.(card8->char 0) (the string value)))
-	     (name (subseq (the string value) 0 name-len))
-	     (class (subseq (the string value) (1+ name-len) (1- (length value)))))
-	(values (and (plusp (length name)) name)
-		(and (plusp (length class)) class))))))
+      (let* ((name-len (position 0 (the (vector card8) value)))
+	     (name (subseq (the (vector card8) value) 0 name-len))
+	     (class (subseq (the (vector card8) value) (1+ name-len) (1- (length value)))))
+	(values (and (plusp (length name)) (map 'string #'card8->char name))
+		(and (plusp (length class)) (map 'string #'card8->char class)))))))
 
 (defun set-wm-class (window resource-name resource-class)
   (declare (type window window)
 	   (type (or null stringable) resource-name resource-class))
-  (set-string-property window :WM_CLASS
-		       (concatenate 'string
-				    (string (or resource-name ""))
-				    #.(make-string 1 :initial-element (card8->char 0))
-				    (string (or resource-class ""))
-				    #.(make-string 1 :initial-element (card8->char 0))))
+  (change-property window :WM_CLASS
+		   (concatenate '(vector card8)
+				(map '(vector card8) #'char->card8
+				     (string (or resource-name "")))
+				#(0)
+				(map '(vector card8) #'char->card8
+				     (string (or resource-class "")))
+				#(0))
+		   :string 8)
   (values))
 
 (defun wm-command (window)
   ;; Returns a list whose car is the command and
   ;; whose cdr is the list of arguments
   (declare (type window window))
-  (declare (values list))
+  (declare (clx-values list))
   (do* ((command-string (get-property window :WM_COMMAND :type :STRING
-				      :result-type 'string :transform #'card8->char))
+				      :result-type '(vector card8)))
 	(command nil)
 	(start 0 (1+ end))
 	(end 0)
 	(len (length command-string)))
        ((>= start len) (nreverse command))
-    (setq end (position #.(card8->char 0) command-string :start start))
-    (push (subseq command-string start end) command)))
+    (setq end (position 0 command-string :start start))
+    (push (map 'string #'card8->char (subseq command-string start end))
+	  command)))
 
 (defsetf wm-command set-wm-command)
 (defun set-wm-command (window command)
@@ -101,13 +104,16 @@
   ;; to recover a lisp command.
   (declare (type window window)
 	   (type list command))
-  (set-string-property
-    window :WM_COMMAND
-    (with-output-to-string (stream)
-      (with-standard-io-syntax
-	(dolist (c command)
-	  (prin1 c stream)
-	  (write-char #.(card8->char 0) stream)))))
+  (change-property window :WM_COMMAND
+		   (apply #'concatenate '(vector card8)
+			  (mapcan #'(lambda (c)
+				      (list (map '(vector card8) #'char->card8
+						 (with-output-to-string (stream)
+						   (with-standard-io-syntax
+						     (prin1 c stream))))
+					    #(0)))
+				  command))
+		   :string 8)
   command)
 
 ;;-----------------------------------------------------------------------------
@@ -128,7 +134,7 @@
 
 (defun wm-hints (window)
   (declare (type window window))
-  (declare (values wm-hints))
+  (declare (clx-values wm-hints))
   (let ((prop (get-property window :WM_HINTS :type :WM_HINTS :result-type 'vector)))
     (when prop
       (decode-wm-hints prop (window-display window)))))
@@ -137,14 +143,14 @@
 (defun set-wm-hints (window wm-hints)
   (declare (type window window)
 	   (type wm-hints wm-hints))
-  (declare (values wm-hints))
+  (declare (clx-values wm-hints))
   (change-property window :WM_HINTS (encode-wm-hints wm-hints) :WM_HINTS 32)
   wm-hints)
 
 (defun decode-wm-hints (vector display)
   (declare (type (simple-vector 9) vector)
 	   (type display display))
-  (declare (values wm-hints))
+  (declare (clx-values wm-hints))
   (let ((input-hint 0)
 	(state-hint 1)
 	(icon-pixmap-hint 2)
@@ -181,7 +187,7 @@
 
 (defun encode-wm-hints (wm-hints)
   (declare (type wm-hints wm-hints))
-  (declare (values simple-vector))
+  (declare (clx-values simple-vector))
   (let ((input-hint         #b1)
 	(state-hint         #b10)
 	(icon-pixmap-hint   #b100)
@@ -225,8 +231,8 @@
 ;; WM_SIZE_HINTS
 
 (def-clx-class (wm-size-hints)
-  (user-specified-position-p nil :type boolean) ;; True when user specified x y
-  (user-specified-size-p nil :type boolean)     ;; True when user specified width height
+  (user-specified-position-p nil :type generalized-boolean) ;; True when user specified x y
+  (user-specified-size-p nil :type generalized-boolean)     ;; True when user specified width height
   (x nil :type (or null int16))			;; Obsolete
   (y nil :type (or null int16))			;; Obsolete
   (width nil :type (or null card16))		;; Obsolete
@@ -242,28 +248,28 @@
   (base-width nil :type (or null card16))
   (base-height nil :type (or null card16))
   (win-gravity nil :type (or null win-gravity))
-  (program-specified-position-p nil :type boolean) ;; True when program specified x y
-  (program-specified-size-p nil :type boolean)     ;; True when program specified width height
+  (program-specified-position-p nil :type generalized-boolean) ;; True when program specified x y
+  (program-specified-size-p nil :type generalized-boolean)     ;; True when program specified width height
   )
 
 
 (defun wm-normal-hints (window)
   (declare (type window window))
-  (declare (values wm-size-hints))
+  (declare (clx-values wm-size-hints))
   (decode-wm-size-hints (get-property window :WM_NORMAL_HINTS :type :WM_SIZE_HINTS :result-type 'vector)))
 
 (defsetf wm-normal-hints set-wm-normal-hints)
 (defun set-wm-normal-hints (window hints)
   (declare (type window window)
 	   (type wm-size-hints hints))
-  (declare (values wm-size-hints))
+  (declare (clx-values wm-size-hints))
   (change-property window :WM_NORMAL_HINTS (encode-wm-size-hints hints) :WM_SIZE_HINTS 32)
   hints)
 
 ;;; OBSOLETE
 (defun wm-zoom-hints (window)
   (declare (type window window))
-  (declare (values wm-size-hints))
+  (declare (clx-values wm-size-hints))
   (decode-wm-size-hints (get-property window :WM_ZOOM_HINTS :type :WM_SIZE_HINTS :result-type 'vector)))
 
 ;;; OBSOLETE
@@ -272,13 +278,13 @@
 (defun set-wm-zoom-hints (window hints)
   (declare (type window window)
 	   (type wm-size-hints hints))
-  (declare (values wm-size-hints))
+  (declare (clx-values wm-size-hints))
   (change-property window :WM_ZOOM_HINTS (encode-wm-size-hints hints) :WM_SIZE_HINTS 32)
   hints)
 
 (defun decode-wm-size-hints (vector)
   (declare (type (or null (simple-vector *)) vector))
-  (declare (values (or null wm-size-hints)))
+  (declare (clx-values (or null wm-size-hints)))
   (when vector
     (let ((flags (aref vector 0))
 	  (hints (make-wm-size-hints)))
@@ -311,8 +317,8 @@
 		(decode-type (member-vector *win-gravity-vector*) (aref vector 17)))))
       ;; Obsolete fields
       (when (or (logbitp 0 flags) (logbitp 2 flags))
-	(setf (wm-size-hints-x hints) (aref vector 1)
-	      (wm-size-hints-y hints) (aref vector 2)))
+	(setf (wm-size-hints-x hints) (card32->int32 (aref vector 1))
+	      (wm-size-hints-y hints) (card32->int32 (aref vector 2))))
       (when (or (logbitp 1 flags) (logbitp 3 flags))
 	(setf (wm-size-hints-width hints) (aref vector 3)
 	      (wm-size-hints-height hints) (aref vector 4)))
@@ -320,7 +326,7 @@
 
 (defun encode-wm-size-hints (hints)
   (declare (type wm-size-hints hints))
-  (declare (values simple-vector))
+  (declare (clx-values simple-vector))
   (let ((vector (make-array 18 :initial-element 0))
 	(flags 0))
     (declare (type (simple-vector 18) vector)
@@ -386,7 +392,7 @@
 
 (defun icon-sizes (window)
   (declare (type window window))
-  (declare (values wm-size-hints))
+  (declare (clx-values wm-size-hints))
   (let ((vector (get-property window :WM_ICON_SIZE :type :WM_ICON_SIZE :result-type 'vector)))
     (declare (type (or null (simple-vector 6)) vector))
     (when vector
@@ -488,8 +494,8 @@
 	   (type (or null list) command)
 	   (type (or null wm-hints) hints)
 	   (type (or null wm-size-hints) normal-hints zoom-hints)
-	   (type boolean user-specified-position-p user-specified-size-p)
-	   (type boolean program-specified-position-p program-specified-size-p)
+	   (type generalized-boolean user-specified-position-p user-specified-size-p)
+	   (type generalized-boolean program-specified-position-p program-specified-size-p)
 	   (type (or null int16) x y)
 	   (type (or null card16) width height min-width min-height max-width max-height width-inc height-inc base-width base-height)
 	   (type (or null win-gravity) win-gravity)
@@ -519,8 +525,7 @@
 	(when icon-x (setf (wm-hints-icon-x wm-hints) icon-x))
 	(when icon-y (setf (wm-hints-icon-y wm-hints) icon-y))
 	(when icon-mask (setf (wm-hints-icon-mask wm-hints) icon-mask))
-	(when window-group
-	  (setf (wm-hints-window-group wm-hints) window-group))
+	(when window-group (setf (wm-hints-window-group wm-hints) window-group))
 	(setf (wm-hints window) wm-hints))
       (when hints (setf (wm-hints window) hints)))
   ;; WM-NORMAL-HINTS
@@ -667,7 +672,7 @@
   (declare (type window window)
 	   (type (member :RGB_DEFAULT_MAP :RGB_BEST_MAP :RGB_RED_MAP
 			 :RGB_GREEN_MAP :RGB_BLUE_MAP) property))
-  (declare (values colormap base-pixel max-color mult-color))
+  (declare (clx-values colormap base-pixel max-color mult-color))
   (let ((prop (get-property window property :type :RGB_COLOR_MAP :result-type 'vector)))
     (declare (type (or null simple-vector) prop))
     (when prop
@@ -688,14 +693,14 @@
 	   (type colormap colormap)
 	   (type pixel base-pixel)
 	   (type color max-color mult-color))
-  (let ((prop (apply #'vector (encode-type colormap colormap)
-		     (encode-type rgb-val (color-red max-color))
-		     (encode-type rgb-val (color-red mult-color))
-		     (encode-type rgb-val (color-green max-color))
-		     (encode-type rgb-val (color-green mult-color))
-		     (encode-type rgb-val (color-blue max-color))
-		     (encode-type rgb-val (color-blue mult-color))
-		     base-pixel)))
+  (let ((prop (vector (encode-type colormap colormap)
+		      (encode-type rgb-val (color-red max-color))
+		      (encode-type rgb-val (color-red mult-color))
+		      (encode-type rgb-val (color-green max-color))
+		      (encode-type rgb-val (color-green mult-color))
+		      (encode-type rgb-val (color-blue max-color))
+		      (encode-type rgb-val (color-blue mult-color))
+		      base-pixel)))
     (change-property window property prop :RGB_COLOR_MAP 32)))
 
 ;;-----------------------------------------------------------------------------
@@ -711,7 +716,7 @@
 	   (type (or null array-index) end)
 	   (type t result-type)			;a sequence type
 	   (type (or null (function (integer) t)) transform))
-  (declare (values sequence type format bytes-after))
+  (declare (clx-values sequence type format bytes-after))
   (let* ((root (screen-root (first (display-roots display))))
 	 (property (aref '#(:CUT_BUFFER0 :CUT_BUFFER1 :CUT_BUFFER2 :CUT_BUFFER3
 			    :CUT_BUFFER4 :CUT_BUFFER5 :CUT_BUFFER6 :CUT_BUFFER7)
@@ -719,36 +724,7 @@
     (get-property root property :type type :result-type result-type
 		  :start start :end end :transform transform)))
 
-;; Implement the following:
-;; (defsetf cut-buffer (display &key (buffer 0) (type :string) (format 8)
-;;			        (transform #'char->card8) (start 0) end) (data)
-;; In order to avoid having to pass positional parameters to set-cut-buffer,
-;; We've got to do the following.  WHAT A PAIN...
-#-clx-ansi-common-lisp
-(define-setf-expander cut-buffer (display &rest option-list)
-  (declare (dynamic-extent option-list))
-  (do* ((options (copy-list option-list))
-	(option options (cddr option))
-	(store (gensym))
-	(dtemp (gensym))
-	(temps (list dtemp))
-	(values (list display)))
-       ((endp option)
-	(values (nreverse temps)
-		(nreverse values)
-		(list store)
-		`(set-cut-buffer ,store ,dtemp ,@options)
-		`(cut-buffer ,@options)))
-    (unless (member (car option) '(:buffer :type :format :start :end :transform))
-      (error "Keyword arg ~s isn't recognized" (car option)))
-    (let ((x (gensym)))
-      (push x temps)
-      (push (cadr option) values)
-      (setf (cadr option) x))))
-
-(defun
-  #+clx-ansi-common-lisp (setf cut-buffer)
-  #-clx-ansi-common-lisp set-cut-buffer
+(defun (setf cut-buffer)
   (data display &key (buffer 0) (type :STRING) (format 8)
 	(start 0) end (transform #'char->card8))
   (declare (type sequence data)
@@ -771,7 +747,7 @@
   ;; When careful-p, ensure all cut-buffer properties are defined, to prevent errors.
   (declare (type display display)
 	   (type int16 delta)
-	   (type boolean careful-p))
+	   (type generalized-boolean careful-p))
   (let* ((root (screen-root (first (display-roots display))))
 	 (buffers '#(:cut_buffer0 :cut_buffer1 :cut_buffer2 :cut_buffer3
 		     :cut_buffer4 :cut_buffer5 :cut_buffer6 :cut_buffer7)))
