@@ -7,7 +7,7 @@
   ;; (:use "COMMON-LISP" "CLOS")
   (:import-from "SYS" "STRING-CONCAT"))
 
-(provide ':clx)
+(provide "clx")
 
 (in-package :xlib)
 
@@ -435,6 +435,9 @@
   `(SET-WINDOW-PRIORITY ,mode ,window ,sibling))
 (defsetf WINDOW-SAVE-UNDER            SET-WINDOW-SAVE-UNDER)
 
+;; for CLUE
+(defsetf GCONTEXT-DISPLAY SET-GCONTEXT-DISPLAY)
+
 
 ;;;; --------------------------------------------------------------------------
 ;;;;  Macros
@@ -1093,8 +1096,8 @@
                          (let ((killid (aref prop 9)))
                            (if (= killid 1)
                                :release-by-freeing-colormap
-                               ;; FIXME function lookup-resource-id not defined
-                               (lookup-resource-id (window-display window) killid)))))))))
+                               (lookup-resource-id (window-display window)
+                                                   killid)))))))))
 
 (defsetf rgb-colormaps set-rgb-colormaps)
 (defun set-rgb-colormaps (window property maps)
@@ -1529,3 +1532,6 @@
 (define-condition unknown-error (request-error) ())
 (define-condition value-error (request-error) ())
 (define-condition window-error (resource-error) ())
+
+(pushnew "XLIB" custom:*system-package-list* :test #'string=)
+(pushnew "XPM" custom:*system-package-list* :test #'string=)
