@@ -14,9 +14,9 @@
 ;
 ; You should have received a copy of the GNU General Public License
 ; along with this program; if not, write to the Free Software Foundation,
-; Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+; Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ;
-; $Id: oracle.lisp,v 1.13 2003/08/27 13:55:12 hin Exp $
+; $Id: oracle.lisp,v 1.23 2005/08/26 13:47:23 hin Exp $
 
 (defpackage "ORACLE"
   (:documentation
@@ -29,6 +29,7 @@
 	   "SQLCOL-NAME" "SQLCOL-TYPE" "SQLCOL-SIZE" "SQLCOL-SCALE" "SQLCOL-PRECISION" "SQLCOL-NULL_OK" ))
 
 (in-package "ORACLE")
+(pushnew :oracle *features*)
 
 ; Use "C" as foreign language
 (default-foreign-language :stdc)
@@ -97,14 +98,16 @@ to-sqlval to-string update-row valid-symbol
 
 ; CONNECT
 
-(defun connect (user password server &optional schema (auto-commit t) (prefetch-buffer-bytes 0) (long-len -1) (truncate-ok t))
-"(ORACLE::CONNECT user password server &optional schema auto-commit prefetch-buffer-bytes long-len truncate-ok)
+(defun connect (user password server &optional schema (auto-commit t) (prefetch-buffer-bytes 0) (long-len -1) truncate-ok connid)
+"(ORACLE::CONNECT user password server &optional schema auto-commit prefetch-buffer-bytes long-len truncate-ok connid)
 
 Connect to an Oracle database.  All subsequent operations will affect
 this database.  A single program can access several different
 databases by repeated calls to CONNECT.  Connections are cached: if
-you call CONNECT again with the same arguments, the actual Oracle
-connection will be re-used.  CONNECT may not be called inside the
+you call CONNECT again with the same user, schema and server, the
+actual Oracle connection will be re-used.  This can be overriden by
+using the CONNID parameter to identify multiple connections to the
+same user, schema and server.  CONNECT may not be called inside the
 WITH-TRANSACTION macro.
 
 Required arguments:
@@ -130,6 +133,8 @@ Optional arguments:
                  will disable long fetching entirely.  If NIL or negative, defaults to
                  500k bytes.
     truncate-ok  If set, allow truncation of LONG columns to long-len (default: NIL).
+    connid       String assigning an ID making the connection unique among other
+                 connections to with the same (user schema sever)
 
 Returns: T if a cached connection was re-used (NIL if a new connection
          was created and cached).
@@ -144,11 +149,23 @@ Returns: T if a cached connection was re-used (NIL if a new connection
   ; Set up global connection cache
   (if (null *oracle-connection-cache*)
       (setf *oracle-connection-cache* (make-hash-table :test #'equal)))
- 
-  ; Construct key for connection cache 
-  (let* ((hkey (connection-key user schema server))
+
+  ; Construct key for connection cache
+  (let* ((hkey (connection-key user schema server connid))
          (conn (gethash hkey *oracle-connection-cache*))
          (result t))
+
+    ; If using cached connection, "ping" it with a query to make sure it's still alive
+    ; If there's an error, disconnect and invalidate; fall through and retry
+    (when conn
+	  (let ((conn-handle (db-connection conn)))
+	    (oracle_exec_sql conn-handle "SELECT 'x' FROM dual" (make-array 0) (c-truth nil))
+	    (when (not (lisp-truth (oracle_success conn-handle)))
+		  (oracle_disconnect conn-handle) ; Don't check for error here
+		  (remhash hkey *oracle-connection-cache*)
+		  (setf conn nil
+				*oracle-connection* nil))))
+
     (when (null conn)
           ; Connect to database
           (let ((handle (oracle_connect user schema password server prefetch-buffer-bytes (c-truth auto-commit) long-len (c-truth truncate-ok))))
@@ -158,9 +175,9 @@ Returns: T if a cached connection was re-used (NIL if a new connection
                    ; Failed all attempts; give up
                   (db-error (oracle_last_error handle)))
             ; OK: cache the connection
-            (setf conn (make-db :connection handle :hkey hkey))
-            (setf (gethash hkey *oracle-connection-cache*) conn)
-            (setf result nil)))
+            (setf conn (make-db :connection handle :hkey hkey)
+				  (gethash hkey *oracle-connection-cache*) conn
+				  result nil)))
     ; Set current connection
     (setf *oracle-connection* conn)
     result))
@@ -228,9 +245,9 @@ for SELECT statements.
                    sql
                    (hash-to-sqlparam-array params)
                    (c-truth (not is-select)))
-  (setf (db-fetch-called *oracle-connection*) nil)
-  (setf (db-pending-row *oracle-connection*) nil)
-  (setf (db-colinfo *oracle-connection*) nil)
+  (setf (db-fetch-called *oracle-connection*) nil
+		(db-pending-row *oracle-connection*) nil
+		(db-colinfo *oracle-connection*) nil)
   (check-success)
 
   ; Get the row count for the result
@@ -288,10 +305,10 @@ when it is inconvenient of impossible to alias the column with
                                                  (list 'do-rows-index-of (list 'quote v))))
                              vars))
           ; Emit the DO loop itself
-          (append (list 
+          (append (list
                    'do*
                    (append (list `(,fetch-result (fetch 'array) (fetch 'array)))
-                             (map 'list 
+                             (map 'list
                                   #'(lambda (k)
                                       (let ((iter (list 'aref-null fetch-result (gethash (to-string (do-rows-var k)) index-vars))))
                                         (list (do-rows-var k) iter iter)))
@@ -344,8 +361,8 @@ Arguments: none
     (cond
      ; Use pending look-ahead row and reset, else do a "real" fetch
      ((db-pending-row *oracle-connection*)
-      (setf result (row-to-result (db-pending-row *oracle-connection*) result-type))
-      (setf (db-pending-row *oracle-connection*) nil))
+      (setf result (row-to-result (db-pending-row *oracle-connection*) result-type)
+			(db-pending-row *oracle-connection*) nil))
 
      ; Check if already at EOF from previous fetches
      ((and (db-fetch-called *oracle-connection*)
@@ -354,7 +371,7 @@ Arguments: none
              oracle-eof))
       ; Do nothing
       )
-       
+
      ; Do a real fetch from Oracle
      (t (let ((fetch_status (oracle_fetch_row (curconn))))
           (cond ((not (lisp-truth fetch_status))
@@ -365,17 +382,17 @@ Arguments: none
                 (t ; Good fetch - get and convert row data
                  (setf result (oracle_row_values (curconn)))
                  (check-success)
-                 
+
                  ; Convert NULL values to NIL
                  (map-into result #'from-sqlval result)
-                   
+
                  ; Convert number and string types to Lisp based on Oracle type
                  (let ((colinfo (oracle_column_info (curconn))))
                    (check-success)
                    (map-into result #'convert-type result colinfo)
                    result))))))
     ; Set the flag that fetch was called at least once.
-    ; This is to avoid further fetch calls to underlying Oracle library 
+    ; This is to avoid further fetch calls to underlying Oracle library
     ; when we are already at EOF.
     (setf (db-fetch-called *oracle-connection*) t)
     (row-to-result result result-type)))
@@ -461,6 +478,12 @@ Arguments: none
         ; Convert C truth to Lisp for export
         (map-into result
                   #'(lambda (col)
+					  ; Oracle identifies FLOAT using special value -127 for scale,
+					  ; (which is irrelevant for floats).  In this case, map to "FLOAT"
+					  ; for type name and NIL for scale.  Precision will be given in bits
+					  ; as ANSI specifies
+					  (when (and (equal (sqlcol-type col) "NUMBER") (= (sqlcol-scale col) -127) (not (= 0 (sqlcol-precision col))))
+						    (setf (sqlcol-type col) "FLOAT"))
                       (setf (sqlcol-null_ok col) (lisp-truth (sqlcol-null_ok col)))
                       col)
                   result)
@@ -715,7 +738,7 @@ Argument: none
                                     "~%The allowed column/variable names are:~%~%" (column-names)
                                     "~%")))
      val)))
-                       
+
 ; ROW-TO-RESULT
 ; Convert fetched row array data to result type
 (defun row-to-result (row result-type)
@@ -744,13 +767,22 @@ Argument: none
 
 ; Convert Oracle type based on sqlcol data type.  Oracle numerics are converted
 ; to the appropriate internal Lisp type using READ-FROM-STRING.  NULL is retained
-; as Lisp NIL, and strings and dates are left as Lisp string.
+; as Lisp NIL, BLOB and BFILE are converted to array of bytes,
+; and strings and dates are left as Lisp string.
 (defun convert-type (val sc)
   (let ((dtype (sqlcol-type sc)))
     (cond ((null val) nil)
+		  ((find dtype '("BLOB" "BFILE") :test #'equal)
+		   (hex-to-byte-array val))
           ((find dtype '("NUMBER" "INTEGER" "FLOAT") :test #'equal)
-           (read-from-string val))
-          ((find dtype '("VARCHAR" "DATE" "CHAR" "VARCHAR2" "LONG" "RAW" "LONG RAW" "BLOB" "CLOB" "BFILE") :test #'equal)
+		   (let ((old-default-format *read-default-float-format*))
+			 ;; Adjust default float format, read, then restore old value
+			 (unwind-protect
+				 (progn
+				   (setf *read-default-float-format* 'double-float)
+				   (read-from-string val))
+			   (setf *read-default-float-format* old-default-format))))
+          ((find dtype '("VARCHAR" "DATE" "CHAR" "VARCHAR2" "LONG" "RAW" "LONG RAW" "CLOB" "ROWID DESC") :test #'equal)
            val)
           (t (db-error (cat "Unsupported data type '" dtype "'"))))))
 
@@ -802,10 +834,13 @@ Argument: none
 
 ; CONNECTION-KEY
 ; Construct key suitable for use in hash table keyed on
-; unique triple of (user, schema, server)
-(defun connection-key (user schema server)
+; unique triple of (user, schema, server) and optional connection ID
+(defun connection-key (user schema server &optional connid)
   ; Use ~-delimited string - pretty disgusting, eh?
-  (cat (string-upcase user) "~" (string-upcase schema) "~" (string-upcase server)))
+  (let ((result (cat (string-upcase user) "~" (string-upcase schema) "~" (string-upcase server))))
+	(when connid
+	  (setf result (cat result "~" connid)))
+	result))
 
 ; PAIRS-TO-HASH
 ; Convert a list of pairs ((key1 val1) (key2 val2) ...) to hash, enforcing key uniqueness
@@ -819,7 +854,7 @@ Argument: none
               (when (not (valid-symbol key)) (db-error (cat "Column or parameter '" key "' is not a valid Lisp symbol name."
                                                          "~%Consider using SELECT ... " key " AS <column alias>")))
               ; Check uniqueness
-              (multiple-value-bind 
+              (multiple-value-bind
                (curval already-there) (gethash key result)
                (when already-there (db-error (cat "Column or parameter '" key
                                                "' appears twice in list of (name, value) pairs,~%first with value '"
@@ -966,7 +1001,7 @@ Argument: none
 
 ; CAT
 ; Concatenate strings
-(defun cat (&rest args) 
+(defun cat (&rest args)
   (apply #'concatenate 'string (mapcar #'to-string (flatten args))))
 
 ; ARRAY-TO-HASH
@@ -1032,6 +1067,33 @@ Argument: none
 (defun c-truth (x)
   (if (lisp-truth x) 1 0))
 
+;; HEX-VALUE -- Get integer value of single upper-case hex digit
+(defun hex-value (h)
+  (cond ((and (char>= h #\A) (char<= h #\F))
+         (+ 10 (- (char-code h) (char-code #\A))))
+        ((and (char>= h #\a) (char<= h #\f))
+         (+ 10 (- (char-code h) (char-code #\a))))
+        ((and (char>= h #\0) (char<= h #\9))
+         (- (char-code h) (char-code #\0)))
+        (t (error "Invalid hex digit"))))
+
+;; HEX-BYTE-VALUE -- Get byte value of pair of hex digits
+(defun hex-byte-value (hh)
+  (+ (* 16 (hex-value (elt hh 0)))
+     (hex-value (elt hh 1))))
+
+;; HEX-TO-BYTE-ARRAY -- Convert hex string to byte array
+(defun hex-to-byte-array (h)
+  (let* ((size (/ (length h) 2))
+         (result (make-array size :element-type '(unsigned-byte 8))))
+    (loop
+     for i from 0 to (1- size) do
+     (let ((offset (* 2 i)))
+       (setf (elt result i)
+             (coerce (hex-byte-value (subseq h offset (+ 2 offset)))
+                     '(unsigned-byte 8)))))
+    result))
+
 ; FLATTEN
 ; Flatten list (lifted from Paul Graham)
 (defun flatten (x)
