@@ -1,16 +1,18 @@
 ;; Module for regular expression searching/matching in CLISP
 ;; Bruno Haible 14.4.1995, 18.4.1995 -- 2003
-;; Sam Steingold 1999-10-28 -- 2003
+;; Sam Steingold 1999-10-28 -- 2005
 
 (defpackage "REGEXP"
   (:documentation
    "POSIX Regular Expressions - matching, compiling, executing.")
-  (:use "LISP")
-  (:export "MATCH" "MATCH-START" "MATCH-END" "MATCH-STRING" "REGEXP-QUOTE"
-           "REGEXP-COMPILE" "REGEXP-EXEC" "REGEXP-SPLIT" "WITH-LOOP-SPLIT"))
+  (:use "COMMON-LISP")
+  (:export #:match #:match-start #:match-end #:match-string #:regexp-quote
+           #:regexp-matcher
+           #:regexp-compile #:regexp-exec #:regexp-split #:with-loop-split))
 
 (in-package "REGEXP")
-(push "REGEXP" ext:*system-package-list*)
+(pushnew "REGEXP" custom:*system-package-list* :test #'string=)
+(pushnew :regexp *features*)
 
 (defstruct (match (:constructor make-match-boa (start end))
                   (:constructor ))
@@ -19,12 +21,12 @@
 ;; The following implementation of MATCH compiles the pattern
 ;; once for every search.
 (defun match-once (pattern string &key (start 0) (end nil)
-                   (extended nil) (ignore-case nil)
-                   (newline nil) (nosub nil)
-                   (notbol nil) (noteol nil))
+                                       (extended nil) (ignore-case nil)
+                                       (newline nil) (nosub nil)
+                                       (notbol nil) (noteol nil))
   (regexp-exec (regexp-compile pattern :extended extended
-                               :ignore-case ignore-case
-                               :newline newline :nosub nosub)
+                                       :ignore-case ignore-case
+                                       :newline newline :nosub nosub)
                string :start start :end end :notbol notbol :noteol noteol))
 
 ;; The following implementation of MATCH compiles the pattern
@@ -41,23 +43,23 @@
   (cons pattern (make-array '(2 2 2 2))))
 
 (defun %match (patternbox string &key (start 0) (end nil)
-               (extended nil) (ignore-case nil)
-               (newline nil) (nosub nil)
-               (notbol nil) (noteol nil))
+                                      (extended nil) (ignore-case nil)
+                                      (newline nil) (nosub nil)
+                                      (notbol nil) (noteol nil))
   ;; Compile the pattern, if not already done.
-  (let ((compiled-pattern (aref (cdr patternbox) (if extended 0 1)
-                                (if ignore-case 0 1) (if newline 0 1)
-                                (if nosub 0 1))))
-    (unless (and compiled-pattern #+ffi(ffi:validp compiled-pattern))
+  (let ((compiled-pattern
+          (aref (cdr patternbox) (if extended 0 1) (if ignore-case 0 1)
+                                 (if newline 0 1) (if nosub 0 1))))
+    (unless (and compiled-pattern #+ffi (ffi:validp compiled-pattern))
       (setq compiled-pattern (regexp-compile (car patternbox)
                                              :extended extended
                                              :ignore-case ignore-case
                                              :newline newline :nosub nosub))
       (setf (aref (cdr patternbox) (if extended 0 1) (if ignore-case 0 1)
-                  (if newline 0 1) (if nosub 0 1))
+                                   (if newline 0 1) (if nosub 0 1))
             compiled-pattern))
     (regexp-exec compiled-pattern string :start start :end end
-                 :notbol notbol :noteol noteol)))
+                                         :notbol notbol :noteol noteol)))
 
 ;; Convert a match (of type MATCH) to a substring.
 (defun match-string (string match)
@@ -87,20 +89,20 @@
     qstring))
 
 (defun regexp-split (pattern string &key (start 0) (end nil)
-                     (extended nil) (ignore-case nil)
-                     (newline nil) (nosub nil)
-                     (notbol nil) (noteol nil))
+                                         (extended nil) (ignore-case nil)
+                                         (newline nil) (nosub nil)
+                                         (notbol nil) (noteol nil))
   "Split the STRING by the regexp PATTERN.
 Return a list of substrings of STRINGS."
   (loop
     :with compiled =
             (if (stringp pattern)
               (regexp-compile pattern :extended extended
-                              :ignore-case ignore-case
-                              :newline newline :nosub nosub)
+                                      :ignore-case ignore-case
+                                      :newline newline :nosub nosub)
               pattern)
     :for match = (regexp-exec compiled string :start start :end end
-                              :notbol notbol :noteol noteol)
+                                              :notbol notbol :noteol noteol)
     :collect
       (make-array (- (if match (match-start match) (length string)) start)
                   :element-type 'character
@@ -120,19 +122,27 @@ The line is split with REGEXP-SPLIT using PATTERN."
   (let ((compiled-pattern (gensym "WLS-")) (line (gensym "WLS-"))
         (nb (gensym "WLS-")) (ne (gensym "WLS-")) (st (gensym "WLS-"))
         (be (gensym "WLS-")) (en (gensym "WLS-")))
-    `(loop
-       :with ,compiled-pattern =
-         (if (stringp ,pattern)
-           (regexp-compile ,pattern :extended ,extended
-                           :ignore-case ,ignore-case
-                           :newline ,newline :nosub ,nosub)
+    `(LOOP
+       :WITH ,compiled-pattern =
+         (IF (STRINGP ,pattern)
+           (REGEXP-COMPILE ,pattern :EXTENDED ,extended
+                                    :IGNORE-CASE ,ignore-case
+                                    :NEWLINE ,newline :NOSUB ,nosub)
            ,pattern)
-       :and ,ne = ,noteol :and ,nb = ,notbol :and ,st = ,stream
-       :and ,be = ,start :and ,en = ,end
-       :and ,var
-       :for ,line = (read-line ,st nil nil)
-       :while ,line
-       :do (setq ,var
-             (regexp-split ,compiled-pattern ,line :start ,be :end ,en
-                           :notbol ,nb :noteol ,ne))
+       :AND ,ne = ,noteol
+       :AND ,nb = ,notbol
+       :AND ,st = ,stream
+       :AND ,be = ,start
+       :AND ,en = ,end
+       :AND ,var
+       :FOR ,line = (READ-LINE ,st NIL NIL)
+       :WHILE ,line
+       :DO (SETQ ,var
+             (REGEXP-SPLIT ,compiled-pattern ,line :START ,be :END ,en
+                                                   :NOTBOL ,nb :NOTEOL ,ne))
       ,@forms)))
+
+(defun regexp-matcher (pattern)
+  "A valid value for *APROPOS-MATCHER* in the UTF-8 locale."
+  (let ((compiled (regexp-compile pattern :extended t :ignore-case t)))
+    (lambda (name) (regexp-exec compiled name :boolean t))))
