@@ -1,14 +1,18 @@
 ;; Module for wildcard matching in CLISP
 ;; Bruno Haible 18.4.1995
-;; Sam Steingold 2001-2004
+;; Sam Steingold 2001-2005
 
 (defpackage "WILDCARD"
-  (:use "FFI" "COMMON-LISP")
-  (:export "MATCH" "WILDCARD-MATCHER"))
+  (:modern t)
+  (:use "COMMON-LISP" "FFI")
+  (:export #:match #:wildcard-matcher))
 (in-package "WILDCARD")
+(pushnew "WILDCARD" custom:*system-package-list* :test #'string=)
+(pushnew :wildcard *features*)
 
 (default-foreign-language :stdc)
 
+(c-lines "#include <config.h>~%")
 (c-lines "#include <fnmatch.h>~%")
 
 (def-call-out fnmatch
