@@ -2,7 +2,7 @@
 ;;;; 1.8.1989, 2.9.1989, 8.10.1989
 
 (in-package "EXT")
-(export '(doseq dohash without-package-lock))
+(export '(doseq dohash without-package-lock memoized))
 
 (export '(#-(or UNIX WIN32) custom::*default-time-zone*
           custom::*system-package-list*)
@@ -16,9 +16,16 @@
 (defun copy-symbol (symbol &optional flag) ;; Common LISP, p. 169
   (let ((sym (make-symbol (symbol-name symbol))))
     (when flag
-      (when (boundp symbol) (sys::set-symbol-value sym (%symbol-value symbol)))
+      (when (boundp symbol) (sys::set-symbol-value sym (symbol-value symbol)))
       (when (fboundp symbol) (sys::%putd sym (symbol-function symbol)))
-      (sys::%putplist sym (copy-list (symbol-plist symbol))))
+      (sys::%putplist sym (copy-list (symbol-plist symbol)))
+      #| ;; No, ANSI CL does not say that the status of the symbol in the
+         ;; global environment is copied as well.
+      (cond ((constantp symbol) (sys::%proclaim-constant sym (symbol-value symbol)))
+            ((sys::special-variable-p symbol) (proclaim `(SPECIAL ,sym)))
+            ((sys::global-symbol-macro-p symbol) (sys::%proclaim-symbol-macro sym)))
+      |#
+      )
     sym))
 
 (let ((gentemp-count 0)) ;; Common LISP, p. 170
@@ -77,24 +84,29 @@
          ,resultform))))
 
 ;;; <HS>/Body/mac_with-package-iterator.html
-(defmacro with-package-iterator ((name pack-list &rest types) &body body)
+(defmacro with-package-iterator (&whole whole-form
+                                 (name pack-list &rest types) &body body)
   (unless types
     (error-of-type 'source-program-error
+      :form whole-form
+      :detail types             ; == NIL
       (TEXT "missing symbol types (~S/~S/~S) in ~S")
       ':internal ':external ':inherited 'with-package-iterator))
   (dolist (symboltype types)
     (case symboltype
       ((:INTERNAL :EXTERNAL :INHERITED))
       (t (error-of-type 'source-program-error
+           :form whole-form
+           :detail symboltype
            (TEXT "~S: flag must be one of the symbols ~S, ~S, ~S, not ~S")
            'with-package-iterator ':internal ':external ':inherited
            symboltype))))
   (let ((iterfun (gensym "WPI")))
-    `(let ((,iterfun (package-iterator-function
-                      ,pack-list ',(remove-duplicates types))))
-       (macrolet ((,name () '(funcall ,iterfun)))
+    `(LET ((,iterfun (PACKAGE-ITERATOR-FUNCTION
+                       ,pack-list ',(remove-duplicates types))))
+       (MACROLET ((,name () '(FUNCALL ,iterfun)))
          ,@body))))
-(defun package-iterator-function (pack-list symbol-types)
+(defun package-iterator-function (pack-list symbol-types) ; ABI
   (let ((iterstates
           (mapcar #'(lambda (pack) (sys::package-iterator pack symbol-types))
                   (if (listp pack-list) pack-list (list pack-list)))))
@@ -117,47 +129,55 @@
     #+ffi "FFI" #+(or) "AFFI" #+screen "SCREEN"))
 
 ;; Unlock the specified packages, execute the BODY, then lock them again.
-(defmacro without-package-lock (packages &body body)
+(defmacro with-no-package-lock-internal (packages &body body)
   (let ((locked-packages (gensym "WOPL-")))
-    `(let ((,locked-packages
-            (remove-if-not #'package-lock
-                           (or ',packages *system-package-list*))))
-       (unwind-protect (progn (setf (package-lock ,locked-packages) nil)
-                              ,@body)
-         (setf (package-lock ,locked-packages) t)))))
+    `(LET ((,locked-packages (REMOVE-IF-NOT #'PACKAGE-LOCK ,packages)))
+       (UNWIND-PROTECT
+         (PROGN
+           (SETF (PACKAGE-LOCK ,locked-packages) NIL)
+           ,@body)
+         (SETF (PACKAGE-LOCK ,locked-packages) T)))))
+(defmacro without-package-lock (packages &body body)
+  ;; NB: This is augmented by similar compiler processing,
+  ;; see c-WITHOUT-PACKAGE-LOCK.
+  `(WITH-NO-PACKAGE-LOCK-INTERNAL
+     ,(if packages `',packages '*SYSTEM-PACKAGE-LIST*)
+     ,@body))
 
 ;;; module administration (Chapter 11.8), CLTL p. 188
 
 (defvar *modules* nil)
 
+; Conversion from module name to string.
+; cl-user::abc -> "ABC", cs-cl-user::abc -> "abc".
 (defun module-name (name)
-  (cond ((symbolp name) (string-downcase (symbol-name name)))
-        ((stringp name) name)
-        (t (error-of-type 'type-error
-             :datum name :expected-type '(or symbol string)
-             (type-error-string)
-             (typecase-error-string 'name '(symbol string))
-             name))))
+  (if (and (symbolp name)
+           (symbol-package name)
+           (package-case-inverted-p (symbol-package name)))
+    (cs-cl:string name)
+    (string name)))
 
 (defun provide (name)
   (setq *modules* (adjoin (module-name name) *modules* :test #'string=)))
 
-(defun require (module-name &optional (pathname nil p-given)
-                &aux (mod-name (module-name module-name)))
-  (unless (member mod-name *modules* :test #'string=)
-    (unless p-given (setq pathname (pathname mod-name)))
-    (let (#+CLISP
-          (*load-paths* (if (null *load-truename*) *load-paths*
-                            (cons (make-pathname :name nil :type nil
-                                                 :defaults *load-truename*)
-                                  *load-paths*)))
+(defun require (module-name &optional (pathname nil p-given))
+  (setq module-name (module-name module-name))
+  (unless (member module-name *modules* :test #'string=)
+    (unless p-given (setq pathname (pathname module-name)))
+    (let (#+CLISP (*load-paths* *load-paths*)
           #-CLISP (*default-pathname-defaults* '#""))
+      #+CLISP (pushnew (merge-pathnames "dynmod/" *lib-directory*) *load-paths*
+                       :test #'equal)
+      #+CLISP (when *load-truename*
+                (pushnew (make-pathname :name nil :type nil
+                                        :defaults *load-truename*)
+                         *load-paths* :test #'equal))
       (if (atom pathname) (load pathname) (mapcar #'load pathname)))))
 
 
 ;;; integer constants (Chapter 12)
 
-;; see file INTLOG.TXT
+;; see file INTLOG.D
 (defconstant boole-clr 0)
 (defconstant boole-set 15)
 (defconstant boole-1 10)
@@ -213,44 +233,81 @@
 
 ;;; functions for lists (Chapter 15)
 
+;; convert the SET represented as a LIST to a SET represented as a HASH-TABLE
+(defun list-to-ht (l0 list &key test test-not key)
+  (let ((n1 (list-length-proper l0))
+        (n2 (list-length-proper list)))
+    (unless test-not
+      (let ((ht-test (case test
+                       (eq 'fasthash-eq)
+                       ((eql nil) 'fasthash-eql)
+                       (equal 'fasthash-equal)
+                       (equalp 'fasthash-equalp))))
+        (when ht-test
+          ;; --- boxers or briefs? ---
+          ;; when is it worthwhile to use HASH-TABLEs as opposed to LISTS?
+          ;; in sequence.d:seq_duplicates() we use a HASH-TABLE
+          ;; when the list is longer than 10
+          ;; here we use the following heuristic,
+          ;; supported by a numeric experiment on Linux
+          (unless (and (> n1 15) (> n2 5))
+            (return-from list-to-ht nil))
+          ;; passed the test, generate the HASH-TABLE
+          (let ((ht (make-hash-table :test ht-test :size n2)))
+            (unless key (setq key #'identity))
+            (do ((tail list (cdr tail)))
+                ((endp tail) ht)
+              (sys::puthash (funcall key (car tail)) ht tail))))))))
+
 ;; Auxiliary version of MEMBER, which applies the :KEY argument also to items
-(defun sys::member1 (item list &rest rest &key test test-not key)
-  (declare (ignore test test-not))
-  (unless key (setq key #'identity))
-  (apply #'member (funcall key item) list rest))
+;; used only by the recursive version
+;(defun sys::member1 (item list &rest rest &key test test-not key)
+;  (declare (ignore test test-not))
+;  (apply #'member (if key (funcall key item) item) list rest))
+
+(macrolet ((member? (item)
+             (let ((i (gensym "ITEM-")) (k (gensym "KEY-")))
+               `(let* ((,i ,item) (,k (if key (funcall key ,i) ,i)))
+                  (if ht2
+                      ;; values are non-NIL tails,
+                      ;; so the first GETHASH value is good enough
+                      (gethash ,k ht2)
+                      (apply #'member ,k list2 rest))))))
 
 (defun union (list1 list2 &rest rest &key test test-not key)
-  (declare (ignore test test-not key))
+  (declare (ignore test test-not))
   #| ; recursive (not suitable for long lists):
   (cond ((endp list1) list2)
         ((apply #'sys::member1 (car list1) list2 rest)
          (apply #'union (cdr list1) list2 rest))
         (t (cons (car list1) (apply #'union (cdr list1) list2 rest))))
   |# ; iterative
-  (let ((list1-filtered '()))
+  (let* ((list1-filtered '())
+         (ht2 #1=(apply #'list-to-ht list1 list2 rest)))
     (dolist (item list1)
-      (unless (apply #'sys::member1 item list2 rest)
+      (unless (member? item)
         (setq list1-filtered (cons item list1-filtered))))
     (nreconc list1-filtered list2)))
 
 (defun nunion (list1 list2 &rest rest &key test test-not key)
-  (declare (ignore test test-not key))
+  (declare (ignore test test-not))
   #| ; recursive (not suitable for long lists):
   (cond ((endp list1) list2)
         ((apply #'sys::member1 (car list1) list2 rest)
          (apply #'nunion (cdr list1) list2 rest))
         (t (rplacd list1 (apply #'nunion (cdr list1) list2 rest))))
   |# ; iterative
-  (let ((first nil) (last nil))
+  (let* ((first nil) (last nil)
+         (ht2 #1#))
     (do ((l list1 (cdr l)))
         ((endp l))
-      (unless (apply #'sys::member1 (car l) list2 rest)
+      (unless (member? (car l))
         (if last (rplacd last l) (setq first l))
         (setq last l)))
     (if last (progn (rplacd last list2) first) list2)))
 
 (defun intersection (list1 list2 &rest rest &key test test-not key)
-  (declare (ignore test test-not key))
+  (declare (ignore test test-not))
   #| ; recursive (not suitable for long lists):
   (cond ((endp list1) nil)
         ((apply #'sys::member1 (car list1) list2 rest)
@@ -258,30 +315,32 @@
                (apply #'intersection (cdr list1) list2 rest)))
         (t (apply #'intersection (cdr list1) list2 rest)))
   |# ; iterative
-  (let ((list1-filtered '()))
+  (let* ((list1-filtered '())
+         (ht2 #1#))
     (dolist (item list1)
-      (when (apply #'sys::member1 item list2 rest)
+      (when (member? item)
         (setq list1-filtered (cons item list1-filtered))))
     (list-nreverse list1-filtered)))
 
 (defun nintersection (list1 list2 &rest rest &key test test-not key)
-  (declare (ignore test test-not key))
+  (declare (ignore test test-not))
   #| ; recursive (not suitable for long lists):
   (cond ((endp list1) nil)
         ((apply #'sys::member1 (car list1) list2 rest)
          (rplacd list1 (apply #'nintersection (cdr list1) list2 rest)))
         (t (apply #'nintersection (cdr list1) list2 rest)))
   |# ; iterative
-  (let ((first nil) (last nil))
+  (let* ((first nil) (last nil)
+         (ht2 #1#))
     (do ((l list1 (cdr l)))
         ((endp l))
-      (when (apply #'sys::member1 (car l) list2 rest)
+      (when (member? (car l))
         (if last (rplacd last l) (setq first l))
         (setq last l)))
     (if last (progn (rplacd last nil) first) nil)))
 
 (defun set-difference (list1 list2 &rest rest &key test test-not key)
-  (declare (ignore test test-not key))
+  (declare (ignore test test-not))
   #| ; recursive (not suitable for long lists):
   (cond ((endp list1) nil)
         ((not (apply #'sys::member1 (car list1) list2 rest))
@@ -289,24 +348,26 @@
                (apply #'set-difference (cdr list1) list2 rest)))
         (t (apply #'set-difference (cdr list1) list2 rest)))
   |# ; iterative
-  (let ((list1-filtered '()))
+  (let* ((list1-filtered '())
+         (ht2 #1#))
     (dolist (item list1)
-      (unless (apply #'sys::member1 item list2 rest)
+      (unless (member? item)
         (setq list1-filtered (cons item list1-filtered))))
     (list-nreverse list1-filtered)))
 
 (defun nset-difference (list1 list2 &rest rest &key test test-not key)
-  (declare (ignore test test-not key))
+  (declare (ignore test test-not))
   #| ; recursive (not suitable for long lists):
   (cond ((endp list1) nil)
         ((not (apply #'sys::member1 (car list1) list2 rest))
          (rplacd list1 (apply #'nset-difference (cdr list1) list2 rest)))
         (t (apply #'nset-difference (cdr list1) list2 rest)))
   |# ; iterative
-  (let ((first nil) (last nil))
+  (let* ((first nil) (last nil)
+         (ht2 #1#))
     (do ((l list1 (cdr l)))
         ((endp l))
-      (unless (apply #'sys::member1 (car l) list2 rest)
+      (unless (member? (car l))
         (if last (rplacd last l) (setq first l))
         (setq last l)))
     (if last (progn (rplacd last nil) first) nil)))
@@ -333,10 +394,13 @@
 ) ; macrolet rev-arg
 
 (defun subsetp (list1 list2 &rest rest &key test test-not key)
-  (declare (ignore test test-not key))
-  (do ((l list1 (cdr l)))
-      ((endp l) t)
-    (if (not (apply #'sys::member1 (car l) list2 rest)) (return nil))))
+  (declare (ignore test test-not))
+  (do* ((l list1 (cdr l))
+        (ht2 #1#))
+       ((endp l) t)
+    (if (not (member? (car l))) (return nil))))
+
+) ; macrolet member?
 
 ;; Like SUBST-IF, only that the substitution element is
 ;; given by a function and must not be a constant
@@ -370,17 +434,31 @@
                ,HTvar)
              ,resultform))))))
 
+;; Different hash code algorithms for the same test.
+(sys::%putd 'ext:fasthash-eq #'eq)
+(sys::%putd 'ext:stablehash-eq #'eq)
+(sys::%putd 'ext:fasthash-eql #'eql)
+(sys::%putd 'ext:stablehash-eql #'eql)
+(sys::%putd 'ext:fasthash-equal #'equal)
+(sys::%putd 'ext:stablehash-equal #'equal)
+
 
 ;;; functions for strings (Chapter 18)
 
 (defun string-trim (character-bag string)
-  (sys::string-both-trim character-bag character-bag string))
+  (sys::string-both-trim character-bag character-bag string nil))
+(defun cs-cl::string-trim (character-bag string)
+  (sys::string-both-trim character-bag character-bag string t))
 
 (defun string-left-trim (character-bag string)
-  (sys::string-both-trim character-bag nil string))
+  (sys::string-both-trim character-bag nil string nil))
+(defun cs-cl::string-left-trim (character-bag string)
+  (sys::string-both-trim character-bag nil string t))
 
 (defun string-right-trim (character-bag string)
-  (sys::string-both-trim nil character-bag string))
+  (sys::string-both-trim nil character-bag string nil))
+(defun cs-cl::string-right-trim (character-bag string)
+  (sys::string-both-trim nil character-bag string t))
 
 
 ;;; functions for pathnames (Chapter 23.1.5)
@@ -414,22 +492,18 @@
       (unless fi (return-from load-lpt-many nil))
       (when *load-verbose*
         (fresh-line) (write-string ";; ")
-        (write-string (TEXT "Loading logical hosts from file "))
-        (princ file)
-        (write-string " ...")
+        (format t (TEXT "Loading logical hosts from file ~A ...") file)
         (terpri))
       (do* ((eof (gensym)) (host (read fi nil eof) (read fi nil eof)))
            ((eq host eof)
             (write-string ";; ")
-            (write-string (TEXT "Loaded file "))
-            (princ file)
+            (format t (TEXT "Loaded file ~A") file)
             (terpri))
         (setq host (string-upcase host))
         (set-logical-pathname-translations host (eval (read fi)))
         (when *load-verbose*
           (fresh-line) (write-string ";; ")
-          (write-string (TEXT "Defined logical host "))
-          (write-string host)
+          (format t (TEXT "Defined logical host ~A") host)
           (terpri))))
     (gethash host *logical-pathname-translations*))
   ;; load a single host from a file, CMUCL-style
@@ -438,11 +512,7 @@
       (unless fi (return-from load-lpt-one nil))
       (when *load-verbose*
         (fresh-line) (write-string ";; ")
-        (write-string (TEXT "Loading logical host "))
-        (write-string host)
-        (write-string (TEXT " from file "))
-        (princ file)
-        (write-string " ..."))
+        (format t (TEXT "Loading logical host ~S from file ~A ...") host file))
       (set-logical-pathname-translations host (read fi))
       (when *load-verbose*
         (write-string (TEXT " done"))
@@ -480,30 +550,30 @@
                               t)))))))))
       (error (TEXT "No translations for logical host ~S found") host)))
   (set-logical-pathname-translations "SYS"
-    '((";*.LISP" "*.lisp") ("*" "*"))))
+    '((";*.LISP" "*.lisp") (";*" "*") ("*" "/*"))))
 
 
 ;;; functions for time (Chapter 25.4.1)
 
-;; help function for macro TIME
+;; help function for macro TIME ; ABI
 (defun %time (new-real1 new-real2 new-run1 new-run2 new-gc1 new-gc2
               new-space1 new-space2 new-gccount
               old-real1 old-real2 old-run1 old-run2 old-gc1 old-gc2
               old-space1 old-space2 old-gccount)
-  (macrolet ((diff4 (val1-n val2-n val1-o val2-o)
+  (macrolet ((diff4 (newval1 newval2 oldval1 oldval2)
                (if (< internal-time-units-per-second 1000000)
                  ;; TIME_1: UNIX_TIMES
-                 `(delta4 ,val1-n ,val2-n ,val1-o ,val2-o 16)
+                 `(delta4 ,newval1 ,newval2 ,oldval1 ,oldval2 16)
                  ;; TIME_2: other UNIX, WIN32
-                 `(+ (* (- ,val1-n ,val1-o) internal-time-units-per-second)
-                     (- ,val2-n ,val2-o)))))
+                 `(+ (* (- ,newval1 ,oldval1) internal-time-units-per-second)
+                     (- ,newval2 ,oldval2)))))
     (let ((Real-Time (diff4 new-real1 new-real2 old-real1 old-real2))
           (Run-Time (diff4 new-run1 new-run2 old-run1 old-run2))
           (GC-Time (diff4 new-gc1 new-gc2 old-gc1 old-gc2))
           (Space (delta4 new-space1 new-space2 old-space1 old-space2 24))
           (GC-Count (- new-gccount old-gccount))
           (stream *trace-output*))
-      (terpri stream)
+      (fresh-line stream)
       (write-string "Real time: " stream)
       (write (float (/ Real-Time internal-time-units-per-second)) :stream stream)
       (write-string " sec." stream)
@@ -520,7 +590,8 @@
         (write-string "GC: " stream) (write GC-Count :stream stream)
         (write-string ", GC time: " stream)
         (write (float (/ GC-Time internal-time-units-per-second)) :stream stream)
-        (write-string " sec." stream)))))
+        (write-string " sec." stream))
+      (elastic-newline stream))))
 
 ;; (sleep N) pause for N seconds. CLTL p. 447
 (defun sleep (time)
@@ -686,10 +757,10 @@
                                     (when (floatp Zeitzone) (setq Zeitzone (rational Zeitzone)))
                                     (or (integerp Zeitzone)
                                         (and (rationalp Zeitzone) (integerp (* 3600 Zeitzone)))))
-                                  (<= -13 Zeitzone 12)))))))
+                                  (<= -24 Zeitzone 24)))))))
     (error-of-type 'error
-      (TEXT "incorrect date: ~S.~S.~S, ~Sh~Sm~Ss, time zone ~S")
-      Tag Monat Jahr Stunde Minute Sekunde Zeitzone))
+      (TEXT "incorrect date: ~S-~S-~S ~S:~S:~S, time zone ~S")
+      Jahr Monat Tag Stunde Minute Sekunde Zeitzone))
   (+ Sekunde
      (* 60 (+ Minute
               (* 60 (+ Stunde Zeitzone
@@ -703,7 +774,7 @@
     #-(or UNIX WIN32)
     (setq time-zone *default-time-zone*
           Sommerzeit (let ((UT (- UT (round (* 3600 time-zone)))))
-                       (multiple-value-bind (UTTag Stunde) (floor (floor UT 3600) 24)
+                       (multiple-value-bind (UTTag Stunde) (floor UT (* 3600 24))
                          (multiple-value-bind (Jahr Jahrtag) (Jahr&Tag UTTag)
                            (funcall *default-dst-check* Jahr Jahrtag Stunde))))
           Zeitzone (- time-zone (if Sommerzeit 1 0)))
@@ -741,6 +812,60 @@
       (intern (subseq str 1) *keyword-package*)
       (intern str))))
 
+; Gibt object in einen String aus, der nach Möglichkeit höchstens max Spalten
+; lang sein soll.
+(defun write-to-short-string (object max)
+  ; Methode: probiere
+  ; level = 0: length = 0,1,2
+  ; level = 1: length = 1,2,3,4
+  ; level = 2: length = 2,...,6
+  ; usw. bis maximal level = 16.
+  ; Dabei level möglichst groß, und bei festem level length möglichst groß.
+  (if (or (numberp object) (symbolp object)) ; von length und level unbeeinflusst?
+    (write-to-string object)
+    (macrolet ((minlength (level) `,level)
+               (maxlength (level) `(* 2 (+ ,level 1))))
+      ; Um level möglist groß zu bekommen, dabei length = minlength wählen.
+      (let* ((level ; Binärsuche nach dem richtigen level
+               (let ((level1 0) (level2 16))
+                 (loop
+                   (when (= (- level2 level1) 1) (return))
+                   (let ((levelm (floor (+ level1 level2) 2)))
+                     (if (<= (string-width (write-to-string object :level levelm :length (minlength levelm))) max)
+                       (setq level1 levelm) ; levelm passt, probiere größere
+                       (setq level2 levelm) ; levelm passt nicht, probiere kleinere
+                 ) ) )
+                 level1
+             ) )
+             (length ; Binärsuche nach dem richtigen length
+               (let ((length1 (minlength level)) (length2 (maxlength level)))
+                 (loop
+                   (when (= (- length2 length1) 1) (return))
+                   (let ((lengthm (floor (+ length1 length2) 2)))
+                     (if (<= (string-width (write-to-string object :level level :length lengthm)) max)
+                       (setq length1 lengthm) ; lengthm passt, probiere größere
+                       (setq length2 lengthm) ; lengthm passt nicht, probiere kleinere
+                 ) ) )
+                 length1
+            )) )
+        (write-to-string object :level level :length length)
+) ) ) )
+
+(defmacro memoized (form)
+  "(MEMOIZED form) memoizes the result of FORM from its first evaluation."
+  `(LET ((MEMORY
+           (IF (EVAL-WHEN (EVAL) T)
+             ',(cons nil nil)
+             ;; Careful: Different expansions of MEMOIZED forms must yield
+             ;; LOAD-TIME-VALUE forms that are not EQ, otherwise compile-file
+             ;; will coalesce these LOAD-TIME-VALUE forms. Therefore here we
+             ;; explicitly cons up the list and don't use backquote.
+             ,(list 'LOAD-TIME-VALUE '(CONS NIL NIL)))))
+     (UNLESS (CAR MEMORY)
+       (SETF (CDR MEMORY) ,form)
+       (SETF (CAR MEMORY) T))
+     (CDR MEMORY)))
+
 ;; *ERROR-HANDLER* should be NIL or a function which accepts the following
 ;; arguments:
 ;;  - NIL (in case of ERROR) or a continue-format-string (in case of CERROR),
