@@ -1,6 +1,6 @@
 ;;; Inspect
 ;;;
-;;; Copyright (C) 2000 by Sam Steingold
+;;; Copyright (C) 2000-2005 by Sam Steingold
 ;;; This is Free Software, covered by the GNU GPL (v2)
 ;;; See http://www.gnu.org/copyleft/gpl.html
 ;;;
@@ -337,7 +337,10 @@ Supplies some HTTP/1.0 headers and calls `with-html-output'."
              :num-slots (length slots)
              :nth-slot (lambda (ii)
                          (let ((slot (nth ii slots)))
-                           (values (clos:slot-value obj slot) slot)))
+                           (values (if (clos:slot-boundp obj slot)
+                                       (clos:slot-value obj slot)
+                                       *inspect-unbound-value*)
+                                   slot)))
              :set-slot (lambda (ii val)
                          (setf (clos:slot-value obj (nth ii slots)) val))
              :self obj :title "structure object"
@@ -483,13 +486,15 @@ Supplies some HTTP/1.0 headers and calls `with-html-output'."
        (force-output))
       (:d (describe (insp-self insp)))
       ((:p :a) (print-inspection insp *terminal-io* frontend))
-      (:e (handler-case (print (inspect-read-clean-eval insp *terminal-io*))
-            (error (err) (format t " *** error: ~s" err))))
+      (:e (handler-case (let ((v (inspect-read-clean-eval insp *terminal-io*)))
+                          (format t "~&~S~%" v))
+            (error (err) (format t " *** error: ~A" err))))
       (:m (handler-case
-              (print (funcall (insp-set-slot insp)
-                              (inspect-read-clean-eval insp *terminal-io*)
-                              (inspect-read-clean-eval insp *terminal-io*)))
-            (error (err) (format t " *** error: ~s" err))))
+              (let ((v (funcall (insp-set-slot insp)
+                                (inspect-read-clean-eval insp *terminal-io*)
+                                (inspect-read-clean-eval insp *terminal-io*))))
+                (format t "~&~S~%" v))
+            (error (err) (format t " *** error: ~A" err))))
       (t (cond ((setq insp (get-insp id com))
                 (print-inspection insp *terminal-io* frontend)
                 (setq id (insp-id insp)))
